@@ -567,7 +567,7 @@ gst_hls_demux_set_current_variant (GstHLSDemux * hlsdemux,
   if (hlsdemux->current_variant != NULL) {
     gint i;
 
-    //#warning FIXME: Synching fragments across variants
+    //#warning FIXME: Syncing fragments across variants
     //  should be done based on media timestamps, and
     //  discont-sequence-numbers not sequence numbers.
     variant->m3u8->sequence_position =
@@ -647,7 +647,7 @@ gst_hls_demux_process_manifest (GstAdaptiveDemux * demux, GstBuffer * buf)
     gst_hls_demux_set_current_variant (hlsdemux, variant);      // FIXME: inline?
   }
 
-  /* get the selected media playlist (unless the inital list was one already) */
+  /* get the selected media playlist (unless the initial list was one already) */
   if (!hlsdemux->master->is_simple) {
     GError *err = NULL;
 
@@ -770,18 +770,25 @@ gst_hls_demux_start_fragment (GstAdaptiveDemux * demux,
   if (key == NULL)
     goto key_failed;
 
-  gst_hls_demux_stream_decrypt_start (hls_stream, key->data,
-      hls_stream->current_iv);
+  if (!gst_hls_demux_stream_decrypt_start (hls_stream, key->data,
+          hls_stream->current_iv))
+    goto decrypt_start_failed;
 
   return TRUE;
 
 key_failed:
   {
-    GST_ELEMENT_ERROR (demux, STREAM, DEMUX,
+    GST_ELEMENT_ERROR (demux, STREAM, DECRYPT_NOKEY,
         ("Couldn't retrieve key for decryption"), (NULL));
     GST_WARNING_OBJECT (demux, "Failed to decrypt data");
     return FALSE;
   }
+decrypt_start_failed:
+  {
+    GST_ELEMENT_ERROR (demux, STREAM, DECRYPT, ("Failed to start decrypt"),
+        ("Couldn't set key and IV or plugin was built without crypto library"));
+    return FALSE;
+  }
 }
 
 static GstHLSTSReaderType
@@ -948,6 +955,9 @@ gst_hls_demux_finish_fragment (GstAdaptiveDemux * demux,
     }
   }
 
+  if (G_UNLIKELY (stream->downloading_header || stream->downloading_index))
+    return GST_FLOW_OK;
+
   gst_hls_demux_stream_clear_pending_data (hls_stream);
 
   if (ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED)
@@ -1088,6 +1098,18 @@ gst_hls_demux_update_fragment_info (GstAdaptiveDemuxStream * stream)
     return GST_FLOW_EOS;
   }
 
+  if (GST_ADAPTIVE_DEMUX_STREAM_NEED_HEADER (stream) && file->init_file) {
+    GstM3U8InitFile *header_file = file->init_file;
+    stream->fragment.header_uri = g_strdup (header_file->uri);
+    stream->fragment.header_range_start = header_file->offset;
+    if (header_file->size != -1) {
+      stream->fragment.header_range_end =
+          header_file->offset + header_file->size - 1;
+    } else {
+      stream->fragment.header_range_end = -1;
+    }
+  }
+
   if (stream->discont)
     discont = TRUE;
 
@@ -1713,7 +1735,7 @@ static gboolean
 gst_hls_demux_stream_decrypt_start (GstHLSDemuxStream * stream,
     const guint8 * key_data, const guint8 * iv_data)
 {
-  aes_set_decrypt_key (&stream->aes_ctx.ctx, 16, key_data);
+  aes128_set_decrypt_key (&stream->aes_ctx.ctx, key_data);
   CBC_SET_IV (&stream->aes_ctx, iv_data);
 
   return TRUE;
@@ -1726,7 +1748,7 @@ decrypt_fragment (GstHLSDemuxStream * stream, gsize length,
   if (length % 16 != 0)
     return FALSE;
 
-  CBC_DECRYPT (&stream->aes_ctx, aes_decrypt, length, decrypted_data,
+  CBC_DECRYPT (&stream->aes_ctx, aes128_decrypt, length, decrypted_data,
       encrypted_data);
 
   return TRUE;
