@@ -40,7 +40,7 @@
 #define MPEGTS_MAX_PACKETSIZE MPEGTS_ATSC_PACKETSIZE
 
 #define MPEGTS_AFC_DISCONTINUITY_FLAG           0x80
-#define MPEGTS_AFC_RANDOM_ACCES_FLAGS           0x40
+#define MPEGTS_AFC_RANDOM_ACCESS_FLAG           0x40
 #define MPEGTS_AFC_ELEMENTARY_STREAM_PRIORITY   0x20
 #define MPEGTS_AFC_PCR_FLAG                     0x10
 #define MPEGTS_AFC_OPCR_FLAG                    0x08
